#![allow(unused_imports)]

#[macro_use]
extern crate log;
extern crate simple_logger;

#[cfg(windows)]
extern crate winapi;

#[cfg(unix)]
extern crate libc;

use std::mem::size_of; // buffer/struct size
use std::ptr::null_mut; // NULL
use std::time::Duration;

use std::mem::MaybeUninit;

// https://crates.io/crates/hwloc

/*
mod windows;
#[cfg(winows)]
use windows::{get_cpu_info, set_process_affinity};
#[cfg(unix)]
mod unix;
#[cfg(unix)]
use unix::{get_cpu_info, set_process_affinity};
*/

#[cfg(windows)]
#[path = "windows.rs"]
mod plattform;

#[cfg(unix)]
#[path = "unix.rs"]
mod plattform;

use plattform::{get_cpu_info, set_process_affinity};

fn main() {
    //let mut state = State::new();
    let cpu_info = get_cpu_info();
    info!("Hello, {:?}!", cpu_info);

    let cpu_info = cpu_info.expect("unable to get CPU infos");

    // https://docs.microsoft.com/en-us/windows/win32/procthread/numa-support

    //let cores_to_use = vec![CoreId(0), CoreId(2), CoreId(4), CoreId(6)];
    let cores_to_use = vec![CoreId(0), CoreId(1), CoreId(2), CoreId(3)];
    //let cores_to_use = vec![CoreId(0), CoreId(1)];
    set_process_affinity("*", &cores_to_use, &cpu_info.all_system_cores);

    let three_secs = Duration::from_secs(3);

    /* Demo the switching
    let proc_handle = unsafe { GetCurrentProcess() };
    for i in 0..cpu_info.unwrap() {
        unsafe { SetProcessAffinityMask(proc_handle, (1 << i) as u32) };
        make_load(three_secs * 3);
    }
    */

    info!("complete");
    std::thread::sleep(three_secs);
}

fn make_load(timeout: Duration) {
    let start = std::time::Instant::now();
    let mut i = 0usize;
    while start.elapsed() < timeout {
        if i < 1024 {
            i += 1;
        } else {
            i = 0;
        }
    }
}

#[derive(Debug)]
pub struct ProcessHandle<T> {
    pid: Pid,
    handle: T,
    name: String,
}

#[derive(Debug)]
pub struct CpuInfo {
    all_system_cores: Vec<CoreId>,
}
impl CpuInfo {
    fn n_cores(&self) -> usize {
        // lpSystemAffinityMask.count_ones() as usize
        self.all_system_cores.len()
    }
}

#[derive(Debug, PartialEq)]
pub struct CoreId(u8);

#[derive(Debug, Clone, PartialEq)]
pub struct Pid(u32);
